## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    crop = NULL
)

## ----install_demo, eval=FALSE-------------------------------------------------
#  if (!require("BiocManager", quietly = TRUE)) install.packages("BiocManager")
#  
#  BiocManager::install("squallms")

## ----library_squallms, message=FALSE------------------------------------------
library(squallms)

## ----run_xcms, message=FALSE--------------------------------------------------
suppressPackageStartupMessages({
    library(dplyr)
    library(tidyr)
    library(tibble)
    library(ggplot2)
    library(xcms)
    library(RaMS)
})
mzML_files <- system.file("extdata", package = "RaMS") %>%
    list.files(full.names = TRUE, pattern = "[A-F].mzML")

register(BPPARAM = SerialParam())
cwp <- CentWaveParam(snthresh = 0, extendLengthMSW = TRUE, integrate = 2)
obp <- ObiwarpParam(binSize = 0.1, response = 1, distFun = "cor_opt")
pdp <- PeakDensityParam(
    sampleGroups = 1:3, bw = 12, minFraction = 0,
    binSize = 0.001, minSamples = 0
)

## Using XCMS version < 4
library(MSnbase)
msexp <- readMSData(mzML_files, msLevel. = 1, mode = "onDisk")
fpp <- FillChromPeaksParam(ppm = 5)
## Using XCMS version 4+ (exact values may differ)
# library(MsExperiment)
# msexp <- readMsExperiment(mzML_files, msLevel. = 1, mode = "onDisk")
# fpp <- ChromPeakAreaParam()

xcms_filled <- msexp %>%
    findChromPeaks(cwp) %>%
    adjustRtime(obp) %>%
    groupChromPeaks(pdp) %>%
    fillChromPeaks(fpp)

## ----save_xcms_obj, eval=FALSE, echo=FALSE------------------------------------
#  # saveRDS(xcms_filled, file="inst/extdata/intro_xcms_filled.rds")

## ----makeXcmsObjFlat----------------------------------------------------------
feat_peak_info <- makeXcmsObjFlat(xcms_filled) %>%
    select(feature, starts_with("mz"), starts_with("rt"), filename, filepath)
feat_peak_info %>%
    mutate(filepath = paste0(substr(filepath, 1, 13), "~")) %>%
    head() %>%
    knitr::kable()

## ----pooled_files_only, eval=FALSE--------------------------------------------
#  ## Not run:
#  feat_peak_info <- feat_peak_info %>%
#      filter(grepl("AB|CD", filename))

## ----extractChromMetrics, warning=FALSE---------------------------------------
msdata <- grabMSdata(unique(feat_peak_info$filepath), verbosity = 0)
shape_metrics <- extractChromMetrics(feat_peak_info, ms1_data = msdata$MS1)
knitr::kable(head(shape_metrics))
shape_metrics %>%
    arrange(desc(med_snr)) %>%
    ggplot() +
    geom_point(aes(x = med_rt, y = med_mz, color = med_cor, size = med_snr), alpha = 0.5) +
    theme_bw()

## ----plotrandompeaks----------------------------------------------------------
set.seed(123)
shape_metrics %>%
    slice_sample(n = 8) %>%
    mutate(mzmin = med_mz - med_mz * 5 / 1e6) %>%
    mutate(mzmax = med_mz + med_mz * 5 / 1e6) %>%
    mutate(rtmin = med_rt - 1) %>%
    mutate(rtmax = med_rt + 1) %>%
    left_join(msdata$MS1, join_by(
        between(y$rt, x$rtmin, x$rtmax),
        between(y$mz, x$mzmin, x$mzmax)
    )) %>%
    qplotMS1data() +
    geom_vline(aes(xintercept = med_rt), color = "red") +
    facet_wrap(~feature, scales = "free", nrow = 2)

## ----labelFeatsManual, eval=interactive()-------------------------------------
#  manual_classes <- labelFeatsManual(feat_peak_info, ms1_data = msdata$MS1)

## ----save_prior_demo_labels, eval=FALSE, echo=FALSE---------------------------
#  # saveRDS(manual_classes, "inst/extdata/intro_manual_labels.rds")

## ----load_prior_manual_labels, eval=!interactive(), echo=FALSE----------------
manual_classes <- readRDS(system.file("extdata", "intro_manual_labels.rds", package = "squallms"))

## ----plotManualFeatWindow, echo=FALSE-----------------------------------------
row_data <- feat_peak_info %>%
    group_by(feature) %>%
    summarise(mzmed = median(mz), rtmed = median(rt)) %>%
    filter(feature == "FT001")
mzbounds <- pmppm(row_data$mzmed, 10)
rtbounds <- row_data$rtmed + c(-1, 1)
eic <- msdata$MS1[mz %between% mzbounds][rt %between% rtbounds] %>% arrange(filename, rt)

plot(eic$rt, eic$int, type = "l", yaxt = "n", ylab = "", xlab = "Retention time")
abline(v = row_data$rtmed, col="red")
title(paste(row_data$feature, round(mzbounds[1], 7)))

## ----pca_theory_plots, echo=FALSE---------------------------------------------
pcaoutput <- feat_peak_info %>%
    filter(feature %in% sprintf("FT%03d", 30:50)) %>%
    pickyPCA(
        ms1_data = msdata$MS1, rt_window_width = 1, ppm_window_width = 5,
        verbosity = 0
    )

tile_gp <- pcaoutput$interp_df %>%
    ggplot() +
    geom_tile(aes(x = approx_rt, y = filename, fill = approx_int)) +
    facet_wrap(~feature, nrow = 4) +
    scale_x_continuous(
        breaks = c(1, 25, 50), labels = c("0", "0.5", "1"),
        expand = expansion()
    ) +
    scale_y_discrete(expand = expansion()) +
    theme(
        axis.text = element_blank(), axis.ticks = element_blank(),
        legend.position = "none"
    ) +
    labs(x = "Scaled retention time", y = "Individual files")
if (interactive()) {
    ggsave(
        plot = tile_gp, filename = "intro_tile_gp.png", width = 6, height = 3,
        units = "in", path = "vignettes", device = "png", dpi = 120
    )
}

pca_gp <- pcaoutput$pcamat %>%
    prcomp() %>%
    .$rotation %>%
    .[, 1:2] %>%
    as.data.frame() %>%
    rownames_to_column("feature") %>%
    ggplot(aes(x = PC1, y = PC2, label = feature, key = feature)) +
    geom_vline(xintercept = 0) +
    geom_hline(yintercept = 0) +
    geom_text() +
    coord_fixed() +
    scale_x_continuous(expand = expansion(0.2)) +
    labs(x = "Principal component 1", y = "Principal component 2") +
    theme_bw()
if (interactive()) {
    ggsave(
        plot = pca_gp, filename = "intro_pca_gp.png", width = 3, height = 4,
        units = "in", path = "vignettes", device = "png", dpi = 120
    )
}

## ----labelFeatsLasso, eval=interactive()--------------------------------------
#  lasso_classes <- labelFeatsLasso(feat_peak_info, ms1_data = msdata$MS1)

## ----save_prior_demo_lassos, eval=FALSE, echo=FALSE---------------------------
#  # saveRDS(lasso_classes, "inst/extdata/intro_lasso_labels.rds")

## ----load_prior_lasso_labels, eval=!interactive(), echo=FALSE-----------------
lasso_classes <- readRDS(system.file("extdata", "intro_lasso_labels.rds", package = "squallms"))

## ----head_lasso_classes-------------------------------------------------------
head(lasso_classes, 10)

## ----manual_relabel_lasso, eval=interactive()---------------------------------
#  manual_classes <- labelFeatsManual(feat_peak_info, ms1_data = msdata$MS1, selection = "Labeled", existing_labels = lasso_classes)

## ----caret_confusion----------------------------------------------------------
suppressPackageStartupMessages(library(caret))
data.frame(manual = manual_classes, lasso = lasso_classes) %>%
    filter(!is.na(manual) & !is.na(lasso))

data.frame(manual = factor(manual_classes), lasso = factor(lasso_classes)) %>%
    filter(!is.na(manual)) %>%
    with(confusionMatrix(lasso, manual, positive = "Good"))

## ----update_existing----------------------------------------------------------
class_labels <- lasso_classes
class_labels[!is.na(manual_classes)] <- manual_classes[!is.na(manual_classes)]

## ----logModelFeatProb, warning=FALSE------------------------------------------
pred_probs <- logModelFeatProb(shape_metrics, class_labels)
if (interactive()) {
    ggsave("intro_model_spread.png",
        plot = last_plot(), device = "png",
        path = "vignettes", width = 6, height = 4, units = "in", dpi = 120
    )
}
hist(pred_probs, breaks = 20, main = NULL, xlab = "Logistic likelihood")

## ----logModelFeatQuality, warning=FALSE---------------------------------------
pred_classes <- logModelFeatQuality(shape_metrics, class_labels, verbosity = 1)

## ----eval=FALSE---------------------------------------------------------------
#  final_xcms <- updateXcmsObjFeats(shape_metrics, class_labels,
#      verbosity = 0,
#      likelihood_threshold = 0.5
#  )

## -----------------------------------------------------------------------------
sessionInfo()

